#!/usr/bin/env python

"A WebStack application for a system configurator."

import WebStack.Generic
import XSLForms.Utils
import XSLForms.Resources
from XSLTools import XSLOutput
import os

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Static import DirectoryResource

# Resource classes.

class ConfiguratorResource(XSLForms.Resources.XSLFormsResource):

    "A resource providing a system configurator."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    encoding = "utf-8"
    template_resources = {
        "configuration" : ["config_template.xhtml", "config_output.xsl"]
        }
    transform_resources = {
        "database" : ["config_database.xsl"]
        }
    document_resources = {
        "database" : "config_database.xml"
        }
    in_page_resources = {
        "cpu" : ("config_output_cpu.xsl", "cpu-node"),
        "memory" : ("config_output_memory.xsl", "memory-node"),
        "hard-disks" : ("config_output_harddisks.xsl", "hard-disks-node"),
        "accessories" : ("config_output_accessories.xsl", "accessories-node")
        }

    def respond_to_form(self, trans, form):

        """
        Respond to a request having the given transaction 'trans' and the given
        'form' information.
        """

        in_page_resource = self.get_in_page_resource(trans)
        parameters = form.get_parameters()
        documents = form.get_documents()
        selectors = form.get_selectors()

        # Ensure the presence of a document.

        if documents.has_key("configuration"):
            configuration = documents["configuration"]
        else:
            configuration = form.new_instance("configuration")

        # Add and remove elements according to the selectors found.

        XSLForms.Utils.add_elements(selectors.get("add-memory-unit"), "memory-unit")
        XSLForms.Utils.remove_elements(selectors.get("remove-memory-unit"))
        XSLForms.Utils.add_elements(selectors.get("add-storage-unit"), "storage-unit")
        XSLForms.Utils.remove_elements(selectors.get("remove-storage-unit"))
        XSLForms.Utils.add_elements(selectors.get("add-hard-disk"), "hard-disk")
        XSLForms.Utils.remove_elements(selectors.get("remove-hard-disk"))

        # Send a response according to certain parameters.
        # When exported, an XML version of the data is returned.

        if parameters.has_key("export"):
            trans.set_content_type(WebStack.Generic.ContentType("text/xml", self.encoding))
            configuration.toStream(trans.get_response_stream(), trans.get_response_stream_encoding())

        # When not exported, the data is transformed to produce a normal Web
        # page.

        else:

            # Transform, adding enumerations/ranges.

            database_xsl = self.prepare_transform("database")
            database_xml = self.prepare_document("database")
            proc = XSLOutput.Processor(database_xsl, references={"database" : database_xml})
            configuration = proc.get_result(configuration)

            # Start the response.

            trans.set_content_type(WebStack.Generic.ContentType("application/xhtml+xml", self.encoding))

            # Define the stylesheet parameters.

            stylesheet_parameters = {}

            # Ensure that an output stylesheet exists.

            if in_page_resource in self.in_page_resources.keys():
                trans_xsl = self.prepare_fragment("configuration", in_page_resource)
                element_path = parameters.get("element-path", [""])[0]
                stylesheet_parameters["element-path"] = element_path
            else:
                trans_xsl = self.prepare_output("configuration")

            # Complete the response.

            self.send_output(trans, [trans_xsl], configuration, stylesheet_parameters)

            #import sys
            #proc = XSLOutput.Processor([trans_xsl], parameters=stylesheet_parameters)
            #proc.send_output(sys.stderr, "iso-8859-1", configuration)

# Site map initialisation.

def get_site():

    "Return a simple Web site resource."

    # Get the main resource and the directory used by the application.

    configurator_resource = ConfiguratorResource()
    directory = configurator_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "styles" : DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"}),
        "scripts" : DirectoryResource(os.path.join(directory, "scripts"), {"js" : "text/javascript"}),
        # Main page:
        "" : configurator_resource,
        # Fragments:
        "cpu" : configurator_resource,
        "memory" : configurator_resource,
        "hard-disks" : configurator_resource,
        "accessories" : configurator_resource
        })

    return resource

# vim: tabstop=4 expandtab shiftwidth=4
